package cz.muni.fi.pb162.ishop;

import java.util.Map;
import java.io.IOException;
import java.io.OutputStream;

/**
 * Toto rozhrani reprezentuje objednavku. Soucasti objednavky
 * je zakaznik, ktery objednavku ucinil, seznam objednadnych
 * vyrobku a pocty kusu.
 * 
 * @author Jan Pomikalek &lt;xpomikal@fi.muni.cz&gt;
 */

public interface Order
{
    /**
     * Prida polozky na objednavku.
     * 
     * @param products seznam vyrobku, ktere se maji pridat
     * na objednavku
     * @throws java.lang.NullPointerException pokud pole 
     * <code>products</code> obsahuje prvek s hodnotou
     * <code>null</code>
     */
    void addItems(Product... products);
    
    /**
     * Prida polozky na objednavku.
     * 
     * @param product typ vyrobku, ktery se ma pridat na objednavku
     * @param count pocet kusu vyrobku, ktery se ma pridat na objednavku
     * @throws java.lang.NullPointerException pokud ma parametr
     * <code>product</code> hodnotu <code>null</code>
     * @throws java.lang.IllegalArgumentException pokud parametr
     * <code>count</code> neobsahuje kladnou hodnotu
     */
    void addItems(Product product, int count);
    
    /**
     * Vrati informace o zakaznikovi.
     * 
     * @return zakaznik, ktery objednavku ucinil
     */
    Customer getCustomer();    
    
    /**
     * Vrati objednane polozky.
     * 
     * @return map: vyrobek -&gt; pocet objednanych kusu
     */
    Map<Product, Integer> getItems();
    
    /**
     * Vrati celkovou cenu objednavky. Vypocte se jako suma cen
     * vsech polozek, kde cena polozky se vypocte jako soucin ceny
     * vyrobku a poctu objednanych kusu.
     * 
     * @return celkova cena objednavky
     */
    double getTotalPrice();
    
    /**
     * Vypise objednavku do vystupniho proudu. V nasledujicim formatu:
     * <pre>
     * jmeno_zakaznika
     * adresa_zakaznika
     * 
     * id_vyrobku_1 nazev_vyrobku_1 cena_za_kus_1 pocet_kusu_1 celkova_cena_1
     * id_vyrobku_2 nazev_vyrobku_2 cena_za_kus_2 pocet_kusu_2 celkova_cena_2
     * ...
     * id_vyrobku_n nazev_vyrobku_n cena_za_kus_n pocet_kusu_n celkova_cena_n
     * 
     * celkova_cena_objednavky
     * </pre>
     * Polozky jsou ve vypisu razeny abecedne podle nazvu vyrobku.
     * 
     * @param os vystupni proud
     */
    void print(OutputStream os) throws IOException;
}
