package cz.muni.fi.pb162.ishop;

import java.util.Map;
import java.util.Set;
import java.io.OutputStream;
import java.io.IOException;

/**
 * Toto rozhrani reprezentuje velmi zjednoduseny model internetoveho
 * obchodu. Obchod umoznuje registrovat nove zakazniky, prijimat
 * dodavky zbozi a zpracovavat objednavky.
 * 
 * @author Jan Pomikalek &lt;xpomikal@fi.muni.cz&gt;
 */

public interface Shop
{
    /**
     * Zaregistruje noveho zakaznika.
     * 
     * @param customer zakaznik, ktery ma byt zaregistrovan
     * @throws java.lang.NullPointerException pokud ma parametr
     * <code>customer</code> hodnotu <code>null</code>
     * @throws cz.muni.fi.pb162.ishop.AlreadyRegisteredException
     * pokud dany zakaznik jiz byl zaregistrovan
     */
    void registerCustomer(Customer customer);
    
    /**
     * Prijme dodavku zbozi. Zbozi se ulozi do skladu. V pripade,
     * ze nelze do skladu ulozit cely obsah dodavky, musi metoda
     * selhat atomicky.
     * 
     * @param supply obsah dodavky, map: vyrobek -&gt; pocet kusu
     * @throws java.lang.NullPointerException pokud ma parametr
     * <code>supply</code> hodnotu <code>null</code>
     * @throws cz.muni.fi.pb162.ishop.StockFullException pokud
     * nelze do skladu ulozit cely obsah dodavky
     */
    void productsSupply(Map<Product, Integer> supply);
    
    /**
     * Zpracuje objednavku. Polozky, ktere jsou na objednavce, se
     * odstrani ze skladu. Nasledne se objednavka vytiskne. V pripade,
     * ze na sklade neni dostatek zbozi pro vyrizeni objednavky,
     * musi metoda selhat atomicky.
     * 
     * @param order objednavka, ktera se ma zpracovat
     * @param os vystupni proud, do ktereho se ma objednavka vytisknout
     * @throws java.lang.NullPointerException pokud ma parametr
     * <code>order</code> nebo <code>os</code> hodnotu <code>null</code>
     * @throws java.io.IOException pokud dojde k chybe vstupu/vystupu
     * @throws cz.muni.fi.pb162.ishop.NotEnoughProductsException
     * pokud na sklade neni dostatek zbozi pro vyrizeni objednavky
     * @throws cz.muni.fi.pb162.ishop.UnregisteredCustomerException
     * neni-li objednatel registrovanym zakaznikem
     */
    void processOrder(Order order, OutputStream os) throws IOException;
}
