package cz.muni.fi.pb162.ishop;

import java.util.Map;
import java.util.HashMap;
import java.io.OutputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.ByteArrayInputStream;

/**
 * Testovaci trida pro balik cz.muni.fi.pb162.ishop.
 *
 * @author Jan Pomikalek &lt;xpomikal@fi.muni.cz&gt;
 */
public class ShopTest extends junit.framework.TestCase
{
    private static final int CUST_ID_OK = 5;
    private static final int CUST_ID_NOT_OK = -5;
    private static final String CUST_NAME_OK = "Josef Novak";
    private static final String CUST_ADDR_OK = "Jindrisska 16, 11150 Praha 1";
    
    private static final int PRODUCT_ID_OK = 3;
    private static final int PRODUCT_ID_NOT_OK = -3;
    private static final String PRODUCT_NAME_OK = "Zelene auticko";
    private static final double PRODUCT_PRICE_OK = 53.25;
    private static final double PRODUCT_PRICE_NOT_OK = 0.0;
    private static final Product.Size PRODUCT_SIZE_OK = Product.Size.MEDIUM;
    
    private static final int STOCK_S_CAPACITY = 20;
    private static final int STOCK_M_CAPACITY = 10;
    private static final int STOCK_L_CAPACITY = 5;
       
    private Product smallProduct;
    private Product mediumProduct;
    private Product largeProduct;
    private Customer customer;
    private Order order;
    private Product product;
    private Stock stock;
    private Shop shop;
    private ByteArrayOutputStream os;

    /**
     * Sets up the test fixture.
     *
     * Called before every test case method.
     */
    protected void setUp()
    {
        smallProduct  = new ProductImpl(1, "small product",  12.50, Product.Size.SMALL);
        mediumProduct = new ProductImpl(2, "medium product", 23.50, Product.Size.MEDIUM);
        largeProduct  = new ProductImpl(3, "large product",  59.90, Product.Size.LARGE);
        customer = new CustomerImpl(CUST_ID_OK, CUST_NAME_OK, CUST_ADDR_OK);
        order = new OrderImpl(customer);
        product = new ProductImpl(PRODUCT_ID_OK, PRODUCT_NAME_OK, PRODUCT_PRICE_OK,
            PRODUCT_SIZE_OK);
        stock = new StockImpl(STOCK_S_CAPACITY, STOCK_M_CAPACITY, STOCK_L_CAPACITY);
        shop = new ShopImpl(stock);
        os = new ByteArrayOutputStream();
    }

    /**
     * Tears down the test fixture.
     *
     * Called after every test case method.
     */
    protected void tearDown()
    {
    }
    
    protected void failExpectedException(String method, String expectedException)
    {
        fail(method + ": nevyhozena ocekavana vyjimka " + expectedException);
    }
    
    protected void failUnexpectedException(String method, Throwable unexpectedException)
    {
        fail(method + ": vyhozena neocekavana vyjimka: " + unexpectedException.getClass());
    }
    
    public void testBasicCustomerExceptions()
    {
        final String constructor = "CustomerImpl";
        
        try
        {
            new CustomerImpl(CUST_ID_NOT_OK, CUST_NAME_OK, CUST_ADDR_OK);
            failExpectedException(constructor, "IllegalArgumentException");
        }
        catch(IllegalArgumentException iae) {}
        catch(Exception e)
        {
            failUnexpectedException(constructor, e);
        }
        
        try
        {
            new CustomerImpl(CUST_ID_OK, null, CUST_ADDR_OK);
            failExpectedException(constructor, "NullPointerException");
        }
        catch(NullPointerException npe) {}
        catch(Exception e)
        {
            failUnexpectedException(constructor, e);
        }
        
        try
        {
            new CustomerImpl(CUST_ID_OK, CUST_NAME_OK, null);
            failExpectedException(constructor, "NullPointerException");
        }
        catch(NullPointerException npe) {}
        catch(Exception e)
        {
            failUnexpectedException(constructor, e);
        }
    }    
    
    public void testBasicOrderExceptions()
    {
        final String constructor = "OrderImpl";
        final String method1 = "addItems";
        
        try
        {
            new OrderImpl(null);
            failExpectedException(constructor, "NullPointerException");
        }
        catch(NullPointerException iae) {}
        catch(Exception e)
        {
            failUnexpectedException(constructor, e);
        }
        
        try
        {
            order.addItems(product, null, product);
            failExpectedException(method1, "NullPointerException");
        }
        catch(NullPointerException npe) {}
        catch(Exception e)
        {
            failUnexpectedException(method1, e);
        }
        
        try
        {
            order.addItems(null, 5);
            failExpectedException(method1, "NullPointerException");
        }
        catch(NullPointerException npe) {}
        catch(Exception e)
        {
            failUnexpectedException(method1, e);
        }
        
        try
        {
            order.addItems(product, 0);
            failExpectedException(method1, "IllegalArgumentException");
        }
        catch(IllegalArgumentException npe) {}
        catch(Exception e)
        {
            failUnexpectedException(method1, e);
        }        
    }
    
    public void testBasicProductExceptions()
    {
        final String constructor = "ProductImpl";
        
        try
        {
            new ProductImpl(PRODUCT_ID_NOT_OK, PRODUCT_NAME_OK,
                PRODUCT_PRICE_OK, PRODUCT_SIZE_OK);
            failExpectedException(constructor, "IllegalArgumentException");
        }
        catch(IllegalArgumentException iae) {}
        catch(Exception e)
        {
            failUnexpectedException(constructor, e);
        }
        
        try
        {
            new ProductImpl(PRODUCT_ID_OK, PRODUCT_NAME_OK,
                PRODUCT_PRICE_NOT_OK, PRODUCT_SIZE_OK);
            failExpectedException(constructor, "IllegalArgumentException");
        }
        catch(IllegalArgumentException iae) {}
        catch(Exception e)
        {
            failUnexpectedException(constructor, e);
        }
        
        try
        {
            new ProductImpl(PRODUCT_ID_OK, null,
                PRODUCT_PRICE_OK, PRODUCT_SIZE_OK);
            failExpectedException(constructor, "NullPointerException");
        }
        catch(NullPointerException iae) {}
        catch(Exception e)
        {
            failUnexpectedException(constructor, e);
        }
        
        try
        {
            new ProductImpl(PRODUCT_ID_OK, PRODUCT_NAME_OK,
                PRODUCT_PRICE_OK, null);
            failExpectedException(constructor, "NullPointerException");
        }
        catch(NullPointerException iae) {}
        catch(Exception e)
        {
            failUnexpectedException(constructor, e);
        }
    }    
    
    public void testBasicShopExceptions()
    {
        final String constructor = "ShopImpl";
        final String method1 = "registerCustomer";
        final String method2 = "productsSupply";
        final String method3 = "processOrder";
        
        try
        {
            new ShopImpl(null);
            failExpectedException(constructor, "NullPointerException");
        }
        catch(NullPointerException iae) {}
        catch(Exception e)
        {
            failUnexpectedException(constructor, e);
        }
        
        try
        {
            shop.registerCustomer(null);
            failExpectedException(method1, "NullPointerException");
        }
        catch(NullPointerException npe) {}
        catch(Exception e)
        {
            failUnexpectedException(method1, e);
        }
        
        try
        {
            shop.productsSupply(null);
            failExpectedException(method2, "NullPointerException");
        }
        catch(NullPointerException npe) {}
        catch(Exception e)
        {
            failUnexpectedException(method2, e);
        }
        
        try
        {
            shop.processOrder(null, os);
            failExpectedException(method3, "NullPointerException");
        }
        catch(NullPointerException npe) {}
        catch(Exception e)
        {
            failUnexpectedException(method3, e);
        }
        
        try
        {
            shop.processOrder(order, null);
            failExpectedException(method3, "NullPointerException");
        }
        catch(NullPointerException npe) {}
        catch(Exception e)
        {
            failUnexpectedException(method3, e);
        }        
    }
    
    public void testCustomerEquals()
    {        
        Customer sameCust = new CustomerImpl(CUST_ID_OK, CUST_NAME_OK, CUST_ADDR_OK);
        Customer diffCust = new CustomerImpl(CUST_ID_OK+1, CUST_NAME_OK, CUST_ADDR_OK);
        
        assertEquals(customer, sameCust);
        assertFalse(customer.equals(diffCust));
    }
    
    public void testProductEquals()
    {        
        Product sameProduct = new ProductImpl(PRODUCT_ID_OK, PRODUCT_NAME_OK,
            PRODUCT_PRICE_OK, PRODUCT_SIZE_OK);
        Product diffProduct = new ProductImpl(PRODUCT_ID_OK+1, PRODUCT_NAME_OK,
            PRODUCT_PRICE_OK, PRODUCT_SIZE_OK);
        
        assertEquals(product, sameProduct);
        assertFalse(product.equals(diffProduct));
    }
    
    public void testGetTotalPrice()
    {
        order.addItems(smallProduct, 3);
        order.addItems(mediumProduct, 5);
        order.addItems(largeProduct);
        
        double expectedPrice =
            smallProduct.getPrice()  * 3 +
            mediumProduct.getPrice() * 5 +
            largeProduct.getPrice();
            
        assertEquals(order.getTotalPrice(), expectedPrice);
    }
       
    public void testPrintOrder() throws IOException
    {                
        order.addItems(smallProduct, 3);
        order.addItems(mediumProduct, 5);
        order.addItems(largeProduct);
        
        final String[] expectedOutput = {
            CUST_NAME_OK,
            CUST_ADDR_OK,
            "",
            largeProduct.getName()  + " " + largeProduct.getPrice()  + " 1 " + 1 * largeProduct.getPrice(),
            mediumProduct.getName() + " " + mediumProduct.getPrice() + " 5 " + 5 * mediumProduct.getPrice(),
            smallProduct.getName()  + " " + smallProduct.getPrice()  + " 3 " + 3 * smallProduct.getPrice(),
            "",
            "" + order.getTotalPrice()
        };
        
        order.print(os);
        BufferedReader reader = new BufferedReader(new InputStreamReader(
            new ByteArrayInputStream(os.toByteArray())));
            
        for (int i=0; i < expectedOutput.length; i++)
        {
            String line = reader.readLine();
            assertEquals("neocekavany vystup na radku " + (i+1) + ":", line, expectedOutput[i]);
        }
    }
    
    public void testRegisterCustomer()
    {
        final String method = "registeredCustomer";
        
        try
        {
            shop.registerCustomer(new CustomerImpl(2, "Tonda", "Tondova adresa"));
            shop.registerCustomer(new CustomerImpl(2, "Pepa", "Pepova adresa"));
            failExpectedException(method, "AlreadyRegisteredException");
        }
        catch(AlreadyRegisteredException arc) {}
        catch(Exception e)
        {
            failUnexpectedException(method, e);
        }
    }
    
    public void testProductsSupply()
    {            
        final String method = "productsSupply";
               
        Map<Product, Integer> supply = new HashMap<Product, Integer>();
        supply.put(smallProduct, STOCK_S_CAPACITY/4);
        supply.put(mediumProduct, STOCK_M_CAPACITY/2);
        supply.put(largeProduct, STOCK_L_CAPACITY/4);
        
        stock.addProduct(smallProduct,  STOCK_S_CAPACITY/2);
        stock.addProduct(mediumProduct, STOCK_M_CAPACITY/2);
        stock.addProduct(largeProduct,  STOCK_L_CAPACITY/2);
        
        shop.productsSupply(supply);
        try
        {
            shop.productsSupply(supply);
            failExpectedException(method, "StockFullException");
        }
        catch(StockFullException sfe) {}
        catch(Exception e)
        {
            failUnexpectedException(method, e);
        }
        
        // atomicke selhani?
        assertEquals(method + ": neselhava atomicky", 
            stock.getProductCount(smallProduct),
            STOCK_S_CAPACITY/2 + STOCK_S_CAPACITY/4);
        assertEquals(method + ": neselhava atomicky", 
            stock.getProductCount(mediumProduct),
            STOCK_M_CAPACITY/2 + STOCK_M_CAPACITY/2);
        assertEquals(method + ": neselhava atomicky", 
            stock.getProductCount(largeProduct),
            STOCK_L_CAPACITY/2 + STOCK_L_CAPACITY/4);
    }
    
    public void testProcessOrder() throws IOException
    {
        final String method = "testProcessOrder";
        
        shop.registerCustomer(customer);
        try
        {
            shop.processOrder(new OrderImpl(new CustomerImpl(10, "Tonda", "Tondova adresa")), os);
            failExpectedException(method, "UnregisteredCustomerException");
        }
        catch(UnregisteredCustomerException uce) {}
        catch(Exception e)
        {
            failUnexpectedException(method, e);
        }
        
        stock.addProduct(smallProduct, STOCK_S_CAPACITY);
        stock.addProduct(mediumProduct, STOCK_M_CAPACITY);
        stock.addProduct(largeProduct, STOCK_L_CAPACITY);
        
        final int sCount = STOCK_S_CAPACITY/3;
        final int mCount = STOCK_M_CAPACITY/2 + STOCK_M_CAPACITY/4;
        final int lCount = STOCK_L_CAPACITY/3;
        order.addItems(smallProduct, sCount);
        order.addItems(mediumProduct, mCount);
        order.addItems(largeProduct, lCount);
        
        shop.processOrder(order, os);
        
        try
        {
            shop.processOrder(order, os);
            failExpectedException(method, "NotEnoughProductsException");
        }
        catch(NotEnoughProductsException nepe) {}
        catch(Exception e)
        {
            failUnexpectedException(method, e);
        }
        
        // atomicke selhani?
        assertEquals(method + ": neselhava atomicky", 
            stock.getProductCount(smallProduct),
            STOCK_S_CAPACITY - sCount);
        assertEquals(method + ": neselhava atomicky", 
            stock.getProductCount(mediumProduct),
            STOCK_M_CAPACITY - mCount);
        assertEquals(method + ": neselhava atomicky", 
            stock.getProductCount(largeProduct),
            STOCK_L_CAPACITY - lCount);
    }
    
    public void testStock()
    {
        final int sCount = STOCK_S_CAPACITY/3;
        final int mCount = STOCK_M_CAPACITY/2 + STOCK_M_CAPACITY/4;
        final int lCount = STOCK_L_CAPACITY/3;
        
        stock.addProduct(smallProduct, sCount);
        stock.addProduct(mediumProduct, mCount);
        stock.addProduct(largeProduct, lCount);
        
        stock.addProduct(smallProduct, sCount);
        stock.addProduct(largeProduct, lCount);
        
        try
        {
            stock.addProduct(mediumProduct, mCount);
            failExpectedException("addProduct", "StockFullException");
        }
        catch(StockFullException sfe) {}
        catch(Exception e)
        {
            failUnexpectedException("addProduct", e);
        }
        
        stock.removeProduct(smallProduct, sCount/2);
        stock.removeProduct(largeProduct, lCount);
        
        try
        {
            stock.removeProduct(mediumProduct, 2*mCount);
            failExpectedException("removeProduct", "NotEnoughProductsException");
        }
        catch(NotEnoughProductsException nepe) {}
        catch(Exception e)
        {
            failUnexpectedException("removeProduct", e);
        }
        
        assertEquals(stock.getProductCount(mediumProduct), mCount);
        assertEquals(stock.getProductCount(largeProduct), lCount);
        
        assertEquals(stock.getSpaceLeft(Product.Size.MEDIUM), STOCK_M_CAPACITY-mCount);
        assertEquals(stock.getSpaceLeft(Product.Size.LARGE),  STOCK_L_CAPACITY-lCount);
    }
}
