package cz.muni.fi.pb162.ishop;

/**
 * <p>
 * Toto rozhrani reprezentuje sklad vyrobku. Do skladu
 * lze vyrobky pridavat a zase je z nej odebirat.
 * </p>
 * 
 * <p>
 * Sklad ma omezenou kapacitu. Pro kazdou ze tri moznych
 * velikosti vyrobku (SMALL, MEDIUM, LARGE) existuje ve
 * skladu prostor, kam se ukladaji pouze vyrobky
 * teto velikosti. Kapacita kazde z techto tri prostor
 * oblasti je dana maximalnim poctem vyrobku, ktere sem
 * lze ulozit. V pripade, ze se naplni prostor pro male
 * vyrobky, nelze uz tedy dalsi maly vyrobek do skladu
 * ulozit, prestoze by v prostoru pro velke vyrobky jeste
 * bylo misto.
 * </p>
 * 
 * @author Jan Pomikalek &lt;xpomikal@fi.muni.cz&gt;
 */

public interface Stock
{   
    /**
     * Vlozi do skladu jeden kus vyrobku.
     * 
     * @param product vyrobek, ktery se ma uskladnit
     * @throws cz.muni.fi.pb162.school.StockFullException
     * pokud ve skladu jiz neni misto pro vyrobek dane velikosti
     * @throws java.lang.NullPointerException pokud ma parametr
     * <code>product</code> hodnotu <code>null</code>
     */
    void addProduct(Product product);

    /**
     * Vlozi do skladu <code>count</code> kusu vyrobku.
     * 
     * @param product typ vyrobku, ktery se ma uskladnit
     * @param count pocet kusu vyrobku, ktere se maji uskladnit
     * @throws cz.muni.fi.pb162.school.StockFullException
     * pokud ve skladu jiz neni misto pro dany pocet vyrobku
     * dane velikosti
     * @throws java.lang.NullPointerException pokud ma parametr
     * <code>product</code> hodnotu <code>null</code>
     * @throws java.lang.IllegalArgumentException pokud parametr
     * <code>count</code> neobsahuje kladnou hodnotu
     */
    void addProduct(Product product, int count);
    
    /**
     * Odstrani ze skladu jeden kus vyrobku.
     * 
     * @param product vyrobek, ktery se ma odstranit ze skladu
     * @throws cz.muni.fi.pb162.school.NotEnoughProductsException
     * pokud na sklade dany vyrobek neni k dispozici
     * @throws java.lang.NullPointerException pokud ma parametr
     * <code>product</code> hodnotu <code>null</code>
     */
    void removeProduct(Product product);
    
    /**
     * Odstrani ze skladu <code>count</code> kusu vyrobku.
     * 
     * @param product typ vyrobku, ktery se ma odstranit ze skladu
     * @param count pocet kusu vyrobku, ktere se maji odstranit ze skladu
     * @throws cz.muni.fi.pb162.school.NotEnoughProductsException
     * pokud na sklade neni k dispozici dany pocet daneho vyrobku
     * @throws java.lang.NullPointerException pokud ma parametr
     * <code>product</code> hodnotu <code>null</code>
     * @throws java.lang.IllegalArgumentException pokud parametr
     * <code>count</code> neobsahuje kladnou hodnotu
     */
    void removeProduct(Product product, int count);
    
    /**
     * Vrati informaci o poctu daneho typu vyrobku na sklade.
     * 
     * @param product vyrobek, jehoz stav na sklade se zjistuje
     * @return pocet kusu vyrobku na sklade
     * @throws java.lang.NullPointerException pokud ma parametr
     * <code>product</code> hodnotu <code>null</code>
     */    
    int getProductCount(Product product);
    
    /**
     * Vrati informaci o volnem miste ve skladu.
     * 
     * @param size velikost vyrobku urcujici oblast skladu, jejiz
     * kapacita se zjistuje
     * @return pocet kusu vyrobku dane velikosti, ktere lze do
     * skladu jeste umistit
     * @throws java.lang.NullPointerException pokud ma parametr
     * <code>size</code> hodnotu <code>null</code>
     */
    int getSpaceLeft(Product.Size size);
}
